@@ -1,6 +1,6 @@
 use strict; use warnings;
 package Inline::Module;
-our $VERSION = '0.28';
+our $VERSION = '0.32';
 our $API_VERSION = 'v2';
 
 use Carp 'croak';
@@ -9,14 +9,13 @@ use File::Find();
 use File::Path();
 use File::Spec();
 
-my $inline_build_path = './blib/Inline';
+my $inline_build_path = '.inline';
 
 use constant DEBUG_ON => $ENV{PERL_INLINE_MODULE_DEBUG} ? 1 : 0;
 sub DEBUG { if (DEBUG_ON) { print "DEBUG >>> ", sprintf(@_), "\n" }}
 
 #------------------------------------------------------------------------------
 # This import serves multiple roles:
-# - -MInline::Module=autostub
 # - ::Inline module's proxy to Inline.pm
 # - Makefile.PL postamble
 # - Makefile rule support
@@ -26,8 +25,9 @@ sub import {
     DEBUG_ON && DEBUG "Inline::Module::import(@_)";
 
     my ($stub_module, $program) = caller;
+    $program =~ s!.*[\\\/]!!;
 
-    if ($program eq 'Makefile.PL' && not -e 'INLINE.h') {
+    if ($program eq "Makefile.PL" and not -e 'INLINE.h') {
         $class->check_inc_inc($program);
         no warnings 'once';
         *MY::postamble = \&postamble;
@@ -35,6 +35,7 @@ sub import {
     }
     elsif ($program eq 'Build.PL') {
         $class->check_inc_inc($program);
+        return;
     }
 
     return unless @_;
@@ -42,14 +43,17 @@ sub import {
 
     return $class->handle_stub($stub_module, @_)
         if $cmd eq 'stub';
-    return $class->handle_autostub(@_)
-        if $cmd eq 'autostub';
+    return $class->handle_makestub(@_)
+        if $cmd eq 'makestub';
     return $class->handle_distdir(@ARGV)
         if $cmd eq 'distdir';
     return $class->handle_fixblib()
         if $cmd eq 'fixblib';
-    return $class->handle_makestub(@_)
-        if $cmd eq 'makestub';
+
+    # TODO: Deprecated 12/26/2014. Remove this in a month.
+    die "Inline::Module 'autostub' no longer supported. " .
+        "Remove this option from PERL5OPT."
+            if $cmd eq 'autostub';
 
     die "Unknown Inline::Module::import argument '$cmd'"
 }
@@ -128,7 +132,14 @@ sub postamble {
     my $stub_modules = $meta->{stub};
     my $included_modules = $class->included_modules($meta);
 
+    if ($meta->{makestub} and not -e 'inc' and not -e 'INLINE.h') {
+        $class->make_stub_modules(@{$meta->{stub}});
+    }
+
     my $section = <<"...";
+clean ::
+\t- \$(RM_RF) $inline_build_path
+
 distdir : distdir_inline
 
 distdir_inline : create_distdir
@@ -169,56 +180,12 @@ sub handle_makestub {
             croak "Unknown 'makestub' argument: '$arg'";
         }
     }
-    my $dest = 'lib';
 
-    for my $module (@modules) {
-        my $code = $class->proxy_module($module);
-        my $path = $class->write_module($dest, $module, $code);
-        print "Created stub module '$path' (Inline::Module $VERSION)\n";
-    }
+    $class->make_stub_modules(@modules);
 
     exit 0;
 }
 
-sub handle_autostub {
-    my ($class, @args) = @_;
-
-    # Don't mess with Perl tools, while using PERL5OPT and autostub:
-    return unless
-        $0 eq '-e' or
-        defined $ENV{_} and $ENV{_} =~ m!/prove[^/]*$!;
-    # Don't autostub in the distdir:
-    return if -e './inc/Inline/Module.pm';
-
-    DEBUG_ON && DEBUG "Inline::Module::autostub(@_)";
-
-    require lib;
-    lib->import('lib');
-
-    my %autostub_modules;
-    for my $arg (@args) {
-        if ($arg =~ m!::!) {
-            $autostub_modules{$arg} = 1;
-        }
-        else {
-            croak "Unknown 'autostub' argument: '$arg'";
-        }
-    }
-
-    push @INC, sub {
-        my ($this, $module) = @_;
-        delete $ENV{PERL5OPT};
-
-        $module =~ s!\.pm$!!;
-        $module =~ s!/!::!g;
-        $autostub_modules{$module} or return;
-
-        my $code = $class->proxy_module($module);
-        open my $fh, '<', \$code;
-        return $fh;
-    }
-}
-
 sub handle_distdir {
     my ($class, $distdir, @args) = @_;
     my $stub_modules = [];
@@ -266,10 +233,12 @@ sub default_meta {
     $meta->{stub} = [ $meta->{stub} ] unless ref $meta->{stub};
     $meta->{ilsm} ||= 'Inline::C';
     $meta->{ilsm} = [ $meta->{ilsm} ] unless ref $meta->{ilsm};
+    $meta->{bundle} = 1 unless defined $meta->{bundle};
 }
 
 sub included_modules {
     my ($class, $meta) = @_;
+    return [] if not $meta->{bundle};
     my $ilsm = $meta->{ilsm};
     my $include = [
         'Inline',
@@ -324,17 +293,24 @@ sub add_to_distdir {
     return $manifest; # return a list of the files added
 }
 
+sub make_stub_modules {
+    my ($class, @modules) = @_;
+
+    for my $module (@modules) {
+        my $code = $class->proxy_module($module);
+        my $path = $class->write_module('lib', $module, $code, 'onchange');
+        if ($path) {
+            print "Created stub module '$path' (Inline::Module $VERSION)\n";
+        }
+    }
+}
+
 sub read_local_module {
     my ($class, $module) = @_;
     eval "require $module; 1" or die $@;
     my $file = $module;
     $file =~ s!::!/!g;
-    my $filepath = $INC{"$file.pm"};
-    open IN, '<', $filepath
-        or die "Can't open '$filepath' for input:\n$!";
-    my $code = do {local $/; <IN>};
-    close IN;
-    return $code;
+    $class->read_file($INC{"$file.pm"});
 }
 
 sub read_share_cpp_config {
@@ -342,20 +318,14 @@ sub read_share_cpp_config {
     require File::Share;
     my $dir = File::Share::dist_dir('Inline-Module');
     my $path = File::Spec->catfile($dir, 'CPPConfig.pm');
-    open IN, '<', $path
-        or die "Can't open '$path' for input:\n$!";
-    my $code = do {local $/; <IN>};
-    close IN;
-    return $code;
+    $class->read_file($path);
 }
 
 sub proxy_module {
     my ($class, $module) = @_;
 
     return <<"...";
-# DO NOT EDIT
-#
-# GENERATED BY: Inline::Module $Inline::Module::VERSION
+# DO NOT EDIT. GENERATED BY: Inline::Module
 #
 # This module is for author-side development only. When this module is shipped
 # to CPAN, it will be automagically replaced with content that does not
@@ -375,25 +345,32 @@ use Inline::Module stub => '$API_VERSION';
 sub dyna_module {
     my ($class, $module) = @_;
     return <<"...";
-# DO NOT EDIT
-#
-# GENERATED BY: Inline::Module $Inline::Module::VERSION
+# DO NOT EDIT. GENERATED BY: Inline::Module $Inline::Module::VERSION
 
 use strict; use warnings;
 package $module;
 use base 'DynaLoader';
 bootstrap $module;
-
 1;
 ...
 
-# XXX - think about this later:
+# TODO: Add XS VERSION checking support:
 # our \$VERSION = '0.0.5';
 # bootstrap $module \$VERSION;
 }
 
+sub read_file {
+    my ($class, $filepath) = @_;
+    open IN, '<', $filepath
+        or die "Can't open '$filepath' for input:\n$!";
+    my $code = do {local $/; <IN>};
+    close IN;
+    return $code;
+}
+
 sub write_module {
-    my ($class, $dest, $module, $code) = @_;
+    my ($class, $dest, $module, $code, $onchange) = @_;
+    $onchange ||= 0;
 
     $code =~ s/\n+__END__\n.*//s;
 
@@ -404,6 +381,10 @@ sub write_module {
     $dirpath =~ s!(.*)/.*!$1!;
     File::Path::mkpath($dirpath);
 
+    return if $onchange and
+        -e $filepath and
+        $class->read_file($filepath) eq $code;
+
     unlink $filepath;
     open OUT, '>', $filepath
         or die "Can't open '$filepath' for output:\n$!";
